@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Michael Lamothe
+ * Copyright (C) 2009 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -17,23 +17,36 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
- 
-#include "channel.hh"
 
-Channel::Channel(Transponder& transponder) : transponder(transponder)
+#include "me-tv.h"
+#include "channel.h"
+#include "data.h"
+#include <string.h>
+
+Channel::Channel()
 {
+	channel_id = 0;
+	profile_id = 0;
+	flags = 0;
 	service_id = 0;
-	default_video_pid = 0;
-	default_audio_pid = 0;
+	memset(&frontend_parameters, 0, sizeof(struct dvb_frontend_parameters));
 }
 
-gboolean Channel::operator ==(const Channel& channel) const
+Glib::ustring Channel::get_text()
 {
-	return channel.service_id == service_id && channel.transponder.get_frequency() == transponder.get_frequency();
+	Glib::ustring result = encode_xml(name);
+	EpgEvent epg_event;
+	
+	if (epg_events.get_current(epg_event))
+	{
+		result += " - ";
+		result += epg_event.get_title();
+	}
+	
+	return result;
 }
 
-gboolean Channel::operator !=(const Channel& channel) const
+guint Channel::get_transponder_frequency()
 {
-	return !(*this == channel);
+	return frontend_parameters.frequency;
 }
-
