@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Michael Lamothe
+ * Copyright (C) 2009 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -17,41 +17,41 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
- 
+
 #ifndef __CHANNEL_H__
 #define __CHANNEL_H__
 
+#define CHANNEL_FLAG_NONE		0
+#define CHANNEL_FLAG_DVB_T		0x01
+#define CHANNEL_FLAG_DVB_C		0x02
+#define CHANNEL_FLAG_DVB_S		0x03
+#define CHANNEL_FLAG_ATSC		0x04
+
 #include <linux/dvb/frontend.h>
-#include "string.hh"
-#include "transponder.h"
-#include <vector>
-
-#define SWITCHFREQ		11700000
-#define LOF_HI			10600000
-#define LOF_LO			9750000
-#define HIGH_BAND(f)	((f)>=SWITCHFREQ) ? true : false
-
-class Transponder;
+#include "epg_events.h"
+#include "dvb_transponder.h"
 
 class Channel
 {
-private:
-	Transponder& transponder;
 public:
-	Channel(Transponder& transponder);
-		
-	String							name;
-	unsigned int					service_id;
-	unsigned int					default_video_pid;
-	unsigned int					default_audio_pid;
-	unsigned int					default_subtitle_pid;
-	std::vector<int>				audio_pids;
-	std::vector<int>				subtitle_pids;
-	
-	Transponder& get_transponder() { return transponder; }
-	
-	gboolean operator ==(const Channel& channel) const;
-	gboolean operator !=(const Channel& channel) const;
+	Channel();
+
+	guint			channel_id;
+	guint			profile_id;
+	Glib::ustring	name;
+	guint			flags;
+	guint			sort_order;
+	Glib::ustring	mrl;
+	EpgEvents		epg_events;
+
+	// DVB Specific
+	guint service_id;
+	struct dvb_frontend_parameters frontend_parameters;
+	guint get_transponder_frequency();
+
+	Glib::ustring get_text();
 };
 
+typedef std::list<Channel> ChannelList;
+
 #endif
