@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Michael Lamothe
+ * Copyright (C) 2009 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -21,29 +21,35 @@
 #ifndef __DVB_DEMUXER_H__
 #define __DVB_DEMUXER_H__
 
-#include <stdint.h>
-#include <linux/dvb/dmx.h>
 #include <sys/poll.h>
-#include <linux/types.h>
-#include "string.hh"
+#include <stdint.h>
 #include <stdlib.h>
+#include <glibmm.h>
+#include <linux/dvb/dmx.h>
+#include <linux/types.h>
 
-class DvbDemuxer
+namespace Dvb
 {
-private:
-	int fd;
-	struct pollfd pfd[1];
+	class Demuxer
+	{
+	private:
+		int fd;
+		struct pollfd pfd[1];
 
-public:
-	DvbDemuxer(const String& device_path);
-	~DvbDemuxer();
+	public:
+		Demuxer(const Glib::ustring& device_path);
+		~Demuxer();
 
-	void set_pes_filter(uint16_t pid, dmx_pes_type_t pestype);		
-	void set_filter(ushort pid, ushort table_id = 0, ushort mask = 0);
-	void set_buffer_size(unsigned int buffer_size);
-	int read_data(unsigned char* buffer, size_t length);
-	void stop();
-	int get_fd() const;
-};
+		void set_pes_filter(uint16_t pid, dmx_pes_type_t pestype);		
+		void set_filter(ushort pid, ushort table_id, ushort mask = 0);
+		void set_buffer_size(unsigned int buffer_size);
+		gint read(unsigned char* buffer, size_t length);
+		gboolean poll(guint timeout);
+		void stop();
+		int get_fd() const;
+	};
+}
+
+typedef std::list<Dvb::Demuxer*> DemuxerList;
 
 #endif
