@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Michael Lamothe
+ * Copyright (C) 2009 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -18,17 +18,19 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
  
-#include "dvb_si.hh"
-#include "log.hh"
-#include "string_utility.hh"
-#include "application.hh"
-#include "libucsi/atsc/types.h"
+#include "dvb_si.h"
+#include "application.h"
+#include "exception.h"
+#include "atsc_text.h"
 
 #define SHORT_EVENT			0x4D
 #define EXTENDED_EVENT		0x4E
 
 #define GPS_EPOCH			315964800
 
+using namespace Dvb;
+using namespace Dvb::SI;
+
 void dump(guchar* buffer, gsize length)
 {
 	for (guint i = 0; i < length; i++)
@@ -36,11 +38,11 @@ void dump(guchar* buffer, gsize length)
 		guchar ch = buffer[i];
 		if (g_ascii_isalnum(ch))
 		{
-			Log::write ("buffer[%d] = 0x%02X; // (%c)", i, ch, ch);
+			g_debug ("buffer[%d] = 0x%02X; // (%c)", i, ch, ch);
 		}
 		else
 		{
-			Log::write ("buffer[%d] = 0x%02X;", i, ch);
+			g_debug ("buffer[%d] = 0x%02X;", i, ch);
 		}
 	}
 }
@@ -104,7 +106,7 @@ static guint32 crc32(const char *data, int len)
     return crc;
 }
 
-Event::Event()
+Dvb::SI::Event::Event()
 {
 	event_id = 0;
 	start_time = 0;
@@ -113,12 +115,12 @@ Event::Event()
 	free_CA_mode = 0;
 }
 
-DvbSectionParser::DvbSectionParser()
+SectionParser::SectionParser()
 {
-	epg_encoding = Application::get_current().get_configuration().get_string_value("epg_encoding");
+	text_encoding = get_application().get_string_configuration_value("text_encoding");
 }
 
-gsize DvbSectionParser::read_section(DvbDemuxer& demuxer)
+gsize SectionParser::read_section(Demuxer& demuxer)
 {
 	gsize bytes_read = demuxer.read(buffer, 3);
 	
@@ -146,7 +148,7 @@ gsize DvbSectionParser::read_section(DvbDemuxer& demuxer)
 	return section_length;
 }
 
-void DvbSectionParser::parse_pas(DvbDemuxer& demuxer, ProgramAssociationSection& section)
+void SectionParser::parse_pas(Demuxer& demuxer, ProgramAssociationSection& section)
 {
 	gsize section_length = read_section(demuxer);
 	guint offset = 8;
@@ -162,7 +164,41 @@ void DvbSectionParser::parse_pas(DvbDemuxer& demuxer, ProgramAssociationSection&
 	}
 }
 
-gsize get_atsc_text(String& string, const guchar* buffer)
+void SectionParser::parse_sds (Demuxer& demuxer, ServiceDescriptionSection& section)
+{
+	gsize section_length = read_section(demuxer);
+	
+	guint offset = 3;
+	section.transport_stream_id = get_bits(buffer + offset, 0, 16);
+	offset += 8;
+	
+	while (offset < section_length - 4)
+	{
+		Service service;
+		service.id = get_bits(buffer + offset, 0, 16);
+		offset += 3;
+		guint descriptors_loop_length = get_bits(buffer + offset, 4, 12);
+		offset += 2;
+		guint descriptors_end_offset = offset + descriptors_loop_length;
+		while (offset < descriptors_end_offset)
+		{
+			guint descriptor_tag = buffer[offset];
+			guint descriptor_length = buffer[offset + 1];
+			if (descriptor_tag == 0x48)
+			{
+				service.type = buffer[offset + 2];
+				guint service_provider_name_length = get_text(service.provider_name, buffer + offset + 3);
+				get_text(service.name, buffer + offset + 3 + service_provider_name_length);
+			}
+
+			offset += descriptor_length + 2;
+		}
+		
+		section.services.push_back(service);
+	}
+}
+
+gsize get_atsc_text(Glib::ustring& string, const guchar* buffer)
 {
 	size_t text_position = 0;
 	gsize offset = 0;
@@ -188,7 +224,7 @@ gsize get_atsc_text(String& string, const guchar* buffer)
 	return offset;
 }
 
-gboolean DvbSectionParser::find_descriptor(uint8_t tag, const unsigned char *buf, int descriptors_loop_len, 
+gboolean SectionParser::find_descriptor(uint8_t tag, const unsigned char *buf, int descriptors_loop_len, 
 	const unsigned char **desc, int *desc_len)
 {
 	while (descriptors_loop_len > 0)
@@ -218,16 +254,16 @@ gboolean DvbSectionParser::find_descriptor(uint8_t tag, const unsigned char *buf
 	return false;
 }
 
-String DvbSectionParser::get_lang_desc(const guchar* buffer)
+Glib::ustring SectionParser::get_lang_desc(const guchar* b)
 {
 	char c[4];
-	String s;
-	memset( mempcpy( c, buffer+2, 3 ), 0, 1 );
+	Glib::ustring s;
+	memset( mempcpy( c, b+2, 3 ), 0, 1 );
 	s = c;
 	return s;
 }
 
-void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section)
+void SectionParser::parse_pms(Demuxer& demuxer, ProgramMapSection& section)
 {	
 	const guchar* desc = NULL;
 	gsize section_length = read_section(demuxer);
@@ -248,7 +284,7 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 		case STREAM_TYPE_MPEG2:
 		case STREAM_TYPE_MPEG4:
 		case STREAM_TYPE_H264:
-			Log::write(_("Video PID: %d"), elementary_pid);
+			g_debug("Video PID: %d", elementary_pid);
 			{
 				VideoStream video_stream;
 				video_stream.pid = elementary_pid;
@@ -258,7 +294,7 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 			break;
 		case 0x03:
 		case 0x04:
-			Log::write(_("Audio PID: %d"), elementary_pid);
+			g_debug("Audio PID: %d", elementary_pid);
 			{				
 				AudioStream stream;
 				stream.pid = elementary_pid;
@@ -277,7 +313,7 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 				int descriptor_loop_length = 0;
 				if (find_descriptor(0x56, buffer + offset + 5, descriptor_length, &desc, &descriptor_loop_length))
 				{
-					Log::write(_("Teletext PID: %d"), elementary_pid);
+					g_debug("Teletext PID: %d", elementary_pid);
 
 					TeletextStream stream;
 					stream.pid = elementary_pid;
@@ -292,7 +328,7 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 						descriptor.magazine_number	= get_bits(desc + descriptor_index + 6, 5, 3);
 						descriptor.page_number		= get_bits(desc + descriptor_index + 7, 0, 8);
 						
-						Log::write("TeleText: Language: '%s', Type: %d, Magazine Number: %d, Page Number: %d",
+						g_debug("TeleText: Language: '%s', Type: %d, Magazine Number: %d, Page Number: %d",
 							descriptor.language.c_str(),
 							descriptor.type,
 							descriptor.magazine_number,
@@ -306,7 +342,7 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 				}
 				else if (find_descriptor (0x59, buffer + offset + 5, descriptor_length, &desc, NULL))
 				{
-					Log::write(_("Subtitle PID: %d"), elementary_pid);
+					g_debug("Subtitle PID: %d", elementary_pid);
 					SubtitleStream stream;
 					stream.pid = elementary_pid;
 					if (descriptor_length > 0)
@@ -318,7 +354,7 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 							stream.composition_page_id	= get_bits(desc + 6, 0, 16);
 							stream.ancillary_page_id	= get_bits(desc + 8, 0, 16);
 							
-							Log::write(
+							g_debug(
 								"Subtitle composition_page_id: %d, ancillary_page_id: %d, language: %s",
 								stream.composition_page_id, stream.ancillary_page_id, stream.language.c_str() );
 						}
@@ -327,12 +363,12 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 				}
 				else if (find_descriptor (0x6A, buffer + offset + 5, descriptor_length, NULL, NULL))
 				{
-					Log::write(_("AC3 PID: %d"), elementary_pid);
+					g_debug("AC3 PID: %d", elementary_pid);
 					AudioStream stream;
 					stream.pid = elementary_pid;
 					stream.is_ac3 = true;
 
-					const guchar* desc = NULL;
+					desc = NULL;
 					if (find_descriptor(0x0A, buffer + offset + 5, descriptor_length, &desc, NULL))
 					{
 						stream.language = get_lang_desc (desc);
@@ -342,13 +378,13 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 			}
 			break;
 		case 0x81:
-			Log::write(_("AC3 PID: %d"), elementary_pid);
+			g_debug("AC3 PID: %d", elementary_pid);
 			{
 				AudioStream stream;
 				stream.pid = elementary_pid;
 				stream.is_ac3 = true;
 			
-				const guchar* desc = NULL;
+				desc = NULL;
 				if (find_descriptor(0x0A, buffer + offset + 5, descriptor_length, &desc, NULL))
 				{
 					stream.language = get_lang_desc (desc);
@@ -364,9 +400,9 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 	}
 }
 
-void DvbSectionParser::parse_psip_mgt(DvbDemuxer& demuxer, MasterGuideTable& table)
+void SectionParser::parse_psip_mgt(Demuxer& demuxer, MasterGuideTable& table)
 {
-	gsize section_length = read_section(demuxer);
+	read_section(demuxer);
 	guint offset = 9;
 	guint tables_defined = get_bits(&buffer[offset], 0, 16);
 	offset += 2;
@@ -384,9 +420,9 @@ void DvbSectionParser::parse_psip_mgt(DvbDemuxer& demuxer, MasterGuideTable& tab
 	}
 }
 
-void DvbSectionParser::parse_psip_eis(DvbDemuxer& demuxer, EventInformationSection& section)
+void SectionParser::parse_psip_eis(Demuxer& demuxer, EventInformationSection& section)
 {
-	gsize section_length = read_section(demuxer);
+	read_section(demuxer);
 	guint offset = 3;
 
 	section.service_id = get_bits(&buffer[offset], 0, 16);
@@ -394,7 +430,7 @@ void DvbSectionParser::parse_psip_eis(DvbDemuxer& demuxer, EventInformationSecti
 	offset += 6;
 	guint num_events_in_section = buffer[offset++];
 	
-	for (int i = 0; i < num_events_in_section; i++)
+	for (guint i = 0; i < num_events_in_section; i++)
 	{
 		Event event;
 
@@ -408,7 +444,9 @@ void DvbSectionParser::parse_psip_eis(DvbDemuxer& demuxer, EventInformationSecti
 		gsize title_length = buffer[offset++];
 		if (title_length > 0)
 		{
-			get_atsc_text (event.title, &buffer[offset++]);
+			EventText event_text;
+			get_atsc_text (event_text.title, &buffer[offset++]);
+			event.texts.push_back(event_text);
 			offset += title_length;
 		}
 		
@@ -419,7 +457,7 @@ void DvbSectionParser::parse_psip_eis(DvbDemuxer& demuxer, EventInformationSecti
 	}
 }
 
-void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& section)
+void SectionParser::parse_eis(Demuxer& demuxer, EventInformationSection& section)
 {
 	gsize section_length = read_section(demuxer);
 	
@@ -460,7 +498,7 @@ void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& s
 		guint event_dur_min =  ((duration >> 12)&0xf)*10+((duration >>  8)&0xf);
 		guint event_dur_sec =  ((duration >>  4)&0xf)*10+((duration      )&0xf);
 
-		event.duration = (event_dur_hour*60 + event_dur_min) * 60;
+		event.duration = (event_dur_hour*60 + event_dur_min) * 60 + event_dur_sec;
 		
 		guint event_start_day = 0;
 		guint event_start_month = 0;
@@ -485,6 +523,7 @@ void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& s
 
 			memset(&t, 0, sizeof(struct tm));
 
+			t.tm_sec	= event_start_sec;
 			t.tm_min	= event_start_min;
 			t.tm_hour	= event_start_hour;
 			t.tm_mday	= event_start_day;
@@ -500,7 +539,7 @@ void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& s
 
 			if (offset > end_descriptor_offset)
 			{
-				throw Exception("ASSERT: offset > end_descriptor_offset");
+				throw Exception(_("ASSERT: offset > end_descriptor_offset"));
 			}
 
 			section.events.push_back( event );
@@ -511,11 +550,11 @@ void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& s
 	
 	if (offset > section_length)
 	{
-		throw Exception("ASSERT: offset > end_section_offset");
+		throw Exception(_("ASSERT: offset > end_section_offset"));
 	}
 }
 
-gsize DvbSectionParser::decode_event_descriptor (const guchar* event_buffer, Event& event)
+gsize SectionParser::decode_event_descriptor (const guchar* event_buffer, Event& event)
 {
 	if (event_buffer[1] == 0)
 	{
@@ -529,9 +568,11 @@ gsize DvbSectionParser::decode_event_descriptor (const guchar* event_buffer, Eve
 	{		
 	case EXTENDED_EVENT:
 		{
-			String language;
-			String title;
-			String description;
+			Glib::ustring language;
+			Glib::ustring title;
+			Glib::ustring description;
+			Glib::ustring temp_description;
+			Glib::ustring temp_title;
 
 			language = get_lang_desc (event_buffer);
 			unsigned int offset = 6;
@@ -539,51 +580,35 @@ gsize DvbSectionParser::decode_event_descriptor (const guchar* event_buffer, Eve
 			offset++;
 			while (length_of_items > offset - 7)
 			{
-				offset += get_text(description, &event_buffer[offset]);			
-				offset += get_text(title, &event_buffer[offset]);
-			}
-			offset += get_text(description, &event_buffer[offset]);
-			
-			if (event.description.is_empty())
-			{
-				event.description = description;
-			}
+				offset += get_text(temp_description, &event_buffer[offset]);			
+				description += temp_description;
 
-			if (event.title.is_empty())
-			{
-				event.title = title;
+				offset += get_text(temp_title, &event_buffer[offset]);
+				title += temp_title;
 			}
+			offset += get_text(temp_description, &event_buffer[offset]);
+			description += temp_description;
+			
+			EventText event_text;
+			event_text.is_extended = true;
+			event_text.language = language;
+			event_text.description += description;
+			event_text.title += title;
+			
+			event.texts.push_back(event_text);
 		}
 		break;
 
 	case SHORT_EVENT:
 		{
-			String language;
-			String title;
-			String description;
-			
-			language = get_lang_desc (event_buffer);
+			EventText event_text;
+			event_text.is_extended = false;
+			event_text.language = get_lang_desc (event_buffer);
 			unsigned int offset = 5;
-			offset += get_text(title, &event_buffer[offset]);
-			offset += get_text(description, &event_buffer[offset]);
-			
-			if (event.title.is_empty())
-			{
-				event.title = title;
-			}
-			else if (event.title != title)
-			{
-				event.title += "-" + title;
-			}
-			
-			if (event.description.is_empty())
-			{
-				event.description = description;
-			}
-			else if (event.description != description)
-			{
-				event.description += "-" + description;
-			}
+			offset += get_text(event_text.title, &event_buffer[offset]);
+			offset += get_text(event_text.description, &event_buffer[offset]);
+					
+			event.texts.push_back(event_text);
 		}
 		break;
 	default:
@@ -593,7 +618,7 @@ gsize DvbSectionParser::decode_event_descriptor (const guchar* event_buffer, Eve
 	return descriptor_length;
 }
 
-guint DvbSectionParser::get_bits(const guchar *buffer, guint bitpos, gsize bitcount)
+guint SectionParser::get_bits(const guchar* b, guint bitpos, gsize bitcount)
 {
 	gsize i;
 	gsize val = 0;
@@ -601,13 +626,13 @@ guint DvbSectionParser::get_bits(const guchar *buffer, guint bitpos, gsize bitco
 	for (i = bitpos; i < bitcount + bitpos; i++)
 	{
 		val = val << 1;
-		val = val + ((buffer[i >> 3] & (0x80 >> (i & 7))) ? 1 : 0);
+		val = val + ((b[i >> 3] & (0x80 >> (i & 7))) ? 1 : 0);
 	}
 	return val;
 }
 
 
-gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
+gsize SectionParser::get_text(Glib::ustring& s, const guchar* text_buffer)
 {
 	gsize length = text_buffer[0];
 	guint text_index = 0;
@@ -620,9 +645,9 @@ gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 		// Skip over length byte
 		index++;
 
-		if (epg_encoding != "auto")
+		if (text_encoding != "auto")
 		{
-			codeset = epg_encoding.c_str();
+			codeset = text_encoding.c_str();
 		}
 		else // Determine codeset from stream
 		{			
@@ -649,7 +674,7 @@ gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 						index++;
 						if (text_buffer[index] != 0x00)
 						{
-							Log::write(_("Warning: second byte of code table id was not 0"));
+							g_warning("Second byte of code table id was not 0");
 						}
 						
 						index++;
@@ -671,8 +696,10 @@ gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 						case 0x0D: codeset = "ISO-8859-13"; break;
 						case 0x0E: codeset = "ISO-8859-14"; break;
 						case 0x0F: codeset = "ISO-8859-15"; break;
+						default: break;
 						}
 					}
+					default: break;
 				}
 				
 				index++;
@@ -681,7 +708,7 @@ gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 		
 		s = "";
 
-		if (epg_encoding == "iso6937")
+		if (text_encoding == "iso6937")
 		{
 			s = convert_iso6937(text_buffer + index, length);
 		}
@@ -693,7 +720,7 @@ gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 				{
 					u_char ch = text_buffer[index];
 
-					if (codeset == "UTF-16BE")
+					if (strcmp(codeset, "UTF-16BE") == 0)
 					{
 						text[text_index++] = ch;
 					}
@@ -735,20 +762,20 @@ gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 
 				if (error != NULL)
 				{
-					String message = String::format(_("Failed to convert to UTF-8: %s"), error->message);
-					Log::write(message);
-					Log::write("Codeset: %s", codeset);
-					Log::write("Length: %d", length);
+					Glib::ustring message = Glib::ustring::compose(_("Failed to convert to UTF-8: %1"), error->message);
+					g_debug("%s", message.c_str());
+					g_debug("Codeset: %s", codeset);
+					g_debug("Length: %zu", length);
 					for (guint i = 0; i < (length+1); i++)
 					{
 						gchar ch = text_buffer[i];
 						if (!isprint(ch))
 						{
-							Log::write("text_buffer[%d]\t= 0x%02X", i, text_buffer[i]);
+							g_debug("text_buffer[%d]\t= 0x%02X", i, text_buffer[i]);
 						}
 						else
 						{
-							Log::write("text_buffer[%d]\t= 0x%02X '%c'", i, text_buffer[i], text_buffer[i]);
+							g_debug("text_buffer[%d]\t= 0x%02X '%c'", i, text_buffer[i], text_buffer[i]);
 						}
 					}
 					throw Exception(message);
@@ -767,9 +794,9 @@ gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 	return length + 1;
 }
 
-String DvbSectionParser::convert_iso6937(const guchar* text_buffer, gsize length)
+Glib::ustring SectionParser::convert_iso6937(const guchar* text_buffer, gsize length)
 {
-	String result;
+	Glib::ustring result;
 	gint val;
 	guint i;
 	
