@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Michael Lamothe
+ * Copyright (C) 2009 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -17,13 +17,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
- 
-#include "engine.hh"
 
-Engine::Engine()
+#include "engine.h"
+#include "application.h"
+#include "me-tv-i18n.h"
+#include <gdk/gdkx.h>
+
+gint Engine::get_window_id()
 {
+	int window_id = GDK_WINDOW_XID(get_drawing_area_video()->get_window()->gobj());
+
+	if (window_id == 0)
+	{
+		throw Exception(_("Window ID was 0"));
+	}
+
+	return window_id;
 }
-	
-Engine::~Engine()
+
+Gtk::DrawingArea* Engine::get_drawing_area_video()
 {
+	return dynamic_cast<Gtk::DrawingArea*>(get_application().get_glade()->get_widget("drawing_area_video"));
 }
