@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Michael Lamothe
+ * Copyright (C) 2009 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -17,33 +17,34 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
- 
+
 #ifndef __ENGINE_H__
 #define __ENGINE_H__
 
-#include <gtk/gtk.h>
-#include <gdk/gdkx.h>
-#include "string.hh"
-#include "io.h"
-
-#define ENGINE_DUAL_LANGUAGE_DISABLE	0
-#define ENGINE_DUAL_LANGUAGE_LEFT		1
-#define ENGINE_DUAL_LANGUAGE_RIGHT		2
+#include "me-tv.h"
+#include <libgnomeuimm.h>
 
 class Engine
 {
 public:
-	Engine();
-	virtual ~Engine();
+	typedef enum
+	{
+		AUDIO_CHANNEL_STATE_BOTH = 0,
+		AUDIO_CHANNEL_STATE_LEFT = 1,
+		AUDIO_CHANNEL_STATE_RIGHT = 2
+	} AudioChannelState;
+
+	virtual ~Engine() {};
 	
-	virtual void write(const gchar* buffer, gsize length) = 0;
-	virtual void open() = 0;
-	virtual void close() = 0;
-	virtual void expose() = 0;
-	virtual void mute(gboolean state) = 0;
-	virtual void set_audio_channel(gint channel) = 0;
-	virtual void set_subtitle_channel(gint channel) = 0;
-	virtual void set_dual_language_state(gint state) = 0;
+	virtual void play(const Glib::ustring& mrl) = 0;
+	virtual void stop() = 0;
+	virtual void set_mute_state(gboolean state) = 0;
+	virtual void set_audio_stream(guint stream) = 0;
+	virtual void set_audio_channel_state(AudioChannelState state) = 0;
+	virtual gboolean is_running() = 0;
+
+	gint get_window_id();
+	Gtk::DrawingArea* get_drawing_area_video();
 };
 
 #endif
