@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Michael Lamothe
+ * Copyright (C) 2009 Michael Lamothe
  *
  * This file is part of Me TV
  *
@@ -21,26 +21,33 @@
 #ifndef __SCHEDULED_RECORDING_DIALOG_H__
 #define __SCHEDULED_RECORDING_DIALOG_H__
 
+#include <libglademm.h>
+#include <libgnomeuimm.h>
+#include "epg_event.h"
+#include "me-tv-ui.h"
 #include "scheduled_recording.h"
-#include <libgnomeui/libgnomeui.h>
 
-class ScheduledRecordingDialog
+class ScheduledRecordingDialog : public Gtk::Dialog
 {
 private:
-	GtkWidget*		dialog_scheduled_recording;
-	GtkEntry*		entry_description;
-	GtkComboBox*	combo_box_channel;
-	GtkComboBox*	combo_box_record_type;
-	GtkComboBox*	combo_box_start_day;
-	GnomeDateEdit*	date_edit_start_date;
-	GtkSpinButton*	spinbutton_start_time_hour;
-	GtkSpinButton*	spinbutton_start_time_minute;
-	GtkSpinButton*	spinbutton_duration;
-	
+	const Glib::RefPtr<Gnome::Glade::Xml> glade;
+
+	Gnome::UI::DateEdit*	date_edit_start_time;
+	Gtk::SpinButton*		spinbutton_duration;
+	Gtk::Entry*				entry_description;
+	ChannelComboBox*		channel_combo_box;
+	guint					scheduled_recording_id;
+
 public:
-	ScheduledRecordingDialog();
-	void show();
-	void show(ScheduledRecording& scheduled_recording);
+	ScheduledRecordingDialog(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& glade);
+
+	static ScheduledRecordingDialog* create(Glib::RefPtr<Gnome::Glade::Xml> glade);
+
+	gint run(Gtk::Window* transient_for, ScheduledRecording& scheduled_recording);
+	gint run(Gtk::Window* transient_for, EpgEvent& epg_event);
+	gint run(Gtk::Window* transient_for, gboolean populate_default = true);
+		
+	ScheduledRecording get_scheduled_recording();
 };
 
 #endif
